% this function try to solve the Joint matrix completion model 4 when having
% missing entries. In this version, the missing entries is ignored, i.e. the
% related weight mu are set to zero.
% this method may get bias when there are many missing entries.
% [New]:        some procedure added to solve the stopping criterion
%   problem.
% =========================================================================
% inputs:
% X:            input signals, ?R^(m*n)*N, where N is total number of signals;
% N:            number of signals;
% omega:	the indexs for missing training entries;
% lambda:        the weight for l1-regularizer term;
% tol:          tolerance for stopping criterion;
% maxIters:     maximal number of iterations;
% limem:        if limen==1, the temporary variable will be cleared to save
% memory;
% =========================================================================
% A:            the common part, which is low rank matrix, ?R^(m*n);
% E:            the innovation part, which is sparse, ?R^(m*n)*N;
% output:       some other output;
% =========================================================================
% version
% 3:            modify the parameters mu and Y, by incorporating C_hat;
% 2:            use weighted average to update;
% 1:            the first version of the code, some parameters tune up;
% =========================================================================
function [C_hat, A_hat, E_hat, output]=JointMC7(X, dim, omega, lambda, tol, maxIter, limem, debug, tau, rho)
%% ========================================================================
StrLen=0;
% check the paramets
disp('check the paramets');
% data size
if length(dim)~=4
    disp('Please input the size of trainig data as [height width people condition]');
    return;
end
m=dim(1);   % height
n=dim(2);   % width
N=dim(3);   % number of people
M=dim(4);   % number of condition
disp(dim);
% showing number of missing entries
disp(length(find(omega(:)~=1)));
% setup lambda
if nargin < 4
lambda = 1 / sqrt(m);
end
disp(lambda);
% tolerance
if nargin < 5
    tol = 1e-7;
elseif tol == -1
    tol = 1e-7;
end
disp(tol);
% max number of iterations
if nargin < 6
    maxIter = 1000;
elseif maxIter == -1
    maxIter = 1000;
end
disp(maxIter);
if nargin < 7
    limem =0;
end
disp(limem);
% % united weight
% if nargin < 7
%     weight=1;
% end
% bebug mode
if nargin < 8
    debug=0;
end
% rho = 1.5;          % this one can be tuned
% tau=1.25;
% rho = 6;          % this one can be tuned
% step size
if nargin < 9
    tau=1.25;
end
if nargin < 10
    rho=1.5;
end
%% ========================================================================
% initialization
disp('initialization');
% parameters
dnorm = norm(X, 'fro');
tolProj = 1e-6 * dnorm;
total_svd = 0;
% variable
C_hat = zeros( m, n*N);
A_hat = zeros( m, n*M);
E_hat = zeros( m*N, n*M);
iter = 0;
converged = false;
stopCriterion = 1;
sv = 5*ones(M, 1);
svp = sv;
% =========================================================================
% common is initialize as the mean of the frames
% C_hat=reshape(mean(reshape(X, m*n, N), 2), m, n);
% Y is the weight for lagarian multiplier
Y=zeros(size(X));
mu=zeros(N, M);
for i=1: N
    for j=1: M
        if omega(i, j)==1
            tmpy=X((i-1)*m+1: i*m, (j-1)*n+1: j*n)-C_hat(:, (i-1)*n+1: i*n);
            try
                norm_two = lansvd(tmpy, 1, 'L');
            catch
                norm_two = norm(tmpy, 2);
            end
            norm_inf = norm( tmpy(:), inf) / lambda;
            dual_norm = max(norm_two, norm_inf);
            tmpy = tmpy / dual_norm;
            Y((i-1)*m+1: i*m, (j-1)*n+1: j*n)=tmpy;
            tmpmu=tau/norm_two;
            mu(i, j)=tmpmu;
        else
            Y((i-1)*m+1: i*m, (j-1)*n+1: j*n)=0;
            mu(i, j)=0;
        end
    end
end
% remove the temporary variable
mu_bar = mu * 1e7;
clear tmpy tmpmu i norm_two norm_inf dual_norm;
%% ========================================================================
% main loop starts here
disp('% main loop starts here');
while ~converged
    stopCriterion=0;
    % outer loop
    iter = iter + 1;
    % solve the primal problem by alternative projection
    sv = sv + round(n * 0.1);
    %% =================================================================
    % part 1
    % part 1: solve E when A is fixed
    % L(E) = lambda * |E|_1 + <Y,D-A-E> + mu/2 *|D-A-E|_F^2;
    for i=1:N
        for j=1: M
	    if omega(i, j)==1
            tmpx=X((i-1)*m+1: i*m, (j-1)*n+1: j*n);
            tmpy=Y((i-1)*m+1: i*m, (j-1)*n+1: j*n);
            tmpa=A_hat(:, (j-1)*n+1: j*n);
            tmpc=C_hat(:, (i-1)*n+1: i*n);
            temp_T = tmpx - tmpc -tmpa + (1/mu(i, j))*tmpy;
            tmpe=max( temp_T - lambda/mu(i, j),0) + min( temp_T + lambda/mu(i, j),0);
            E_hat((i-1)*m+1: i*m, (j-1)*n+1: j*n)=tmpe;
        else
            E_hat((i-1)*m+1: i*m, (j-1)*n+1: j*n)=0;
	    end
        end
    end
    if limem
        clear i tmpx tmpy tmpe tmpc tmpa temp_T;
    end
    %% ================================================================
    %% here
    % part 2: low rank part
    % A_i=argmin_(A_i ) |A_i |_*+?|(X_i-C)-A_i |_F+<Y,X_i-C-A_i>
    for j=1:M
        temp_T=zeros(m, n);
        for i=1: N 
            tmpx=X((i-1)*m+1: i*m, (j-1)*n+1: j*n);
            tmpy=Y((i-1)*m+1: i*m, (j-1)*n+1: j*n);
            tmpe=E_hat((i-1)*m+1: i*m, (j-1)*n+1: j*n);
            tmpc=C_hat(:, (i-1)*n+1: i*n);
            temp_T = temp_T+mu(i, j)*(tmpx - tmpc -tmpe) + tmpy;
        end
        % do an averaging
        temp_T=reshape(mean(reshape(temp_T, m*n, []), 2), m, [])/sum(mu(:, j));
        if choosvd(n, sv(j)) == 1
            [U S V] = lansvd(temp_T, sv(j), 'L');
        else
            [U S V] = svd(temp_T, 'econ');
        end
        diagS = diag(S);
        % the threshold is alpha/sum(mu_i)
        svp(j) = length(find(diagS > N/sum(mu(:, j))));
        if svp(j) < sv(j)
            sv(j) = min(svp(j) + 1, n);
        else
            sv(j) = min(svp(j) + round(0.05*n), n);
        end
        tmpa = U(:,1:svp(j))*diag(diagS(1:svp(j))-N/sum(mu(:, j)))*V(:,1:svp(j))';    
        A_hat(:, (j-1)*n+1: j*n)=tmpa;
    end
    if limem
        clear i tmpx tmpy tmpe tempc tmpa temp_T U S V;
    end
    %% ================================================================
    % part 3: common part
    % C=argmin_C ?_i|X_i-C-A_i |_F +<Y,X_i-C-A_i>
    % solution is C=1/n ?_i?X_i^*+Y?
    for i=1: N
        C_hat(:, (i-1)*n+1: i*n)=0;
        for j=1: M
            tmpx=X((i-1)*m+1: i*m, (j-1)*n+1: j*n);
            tmpy=Y((i-1)*m+1: i*m, (j-1)*n+1: j*n);
            tmpa=A_hat(:, (j-1)*n+1: j*n);
            tmpe=E_hat((i-1)*m+1: i*m, (j-1)*n+1: j*n);
            temp_T = mu(i, j)*(tmpx - tmpe -tmpa)+tmpy;
            C_hat(:, (i-1)*n+1: i*n)=C_hat(:, (i-1)*n+1: i*n)+temp_T;
        end
        C_hat(:, (i-1)*n+1: i*n)=C_hat(:, (i-1)*n+1: i*n)/sum(mu(i, :));
    end
    if limem
        clear i tmpx tmpy tmpe tmpa temp_T;
    end
    %% ====================================================================
    % paprt 4: update the parameters
    total_svd = total_svd + 1;
    Z=zeros(size(X));
    % update Y
    for i=1: N
        for j=1: M
            Z((i-1)*m+1: i*m, (j-1)*n+1: j*n)=...
                X((i-1)*m+1: i*m, (j-1)*n+1: j*n)-...
                C_hat(:, (i-1)*n+1: i*n)-...
                A_hat(:, (j-1)*n+1: j*n)-...
                E_hat((i-1)*m+1: i*m, (j-1)*n+1: j*n);
            % add this part to solve the stop criterion. For missing
            % training entries, we set Z to 0 to match the X being 0.
            Z((i-1)*m+1: i*m, (j-1)*n+1: j*n)=Z((i-1)*m+1: i*m, (j-1)*n+1: j*n)*...
                omega(i, j);
            Y((i-1)*m+1: i*m, (j-1)*n+1: j*n)=...
                Y((i-1)*m+1: i*m, (j-1)*n+1: j*n)+...
                mu(i, j)*Z((i-1)*m+1: i*m, (j-1)*n+1: j*n);
        end
    end
    if limem
        clear i j;
    end
    % some modification here
    % update mu: mu=tau/|X_i-C_hat|_2
    % however, the modification seems to be wrong
    % mu=rho*mu;
    mu = min(mu*rho, mu_bar);
    %% ====================================================================
    % stop Criterion    
    stopCriterion = norm(Z, 'fro')/ dnorm;
    if stopCriterion < tol
        converged = true;
    end    

    str=sprintf('Iteration %d | #svd %d | stopCriterion %f\n',...
        iter, total_svd, stopCriterion);
    fprintf(1, '%s', str);
    StrLen=length(str);
    
    if ~converged && iter >= maxIter
        disp('Maximum iterations reached') ;
        converged = 1 ;       
    end
    clear Z;
    if bitget(debug,1)
        save(['data' num2str(iter, '%03d') '.mat']);
    end
    % =====================================================================
end
%% ========================================================================
output.svp=svp;
output.iter=iter;
output.stopCriterion=stopCriterion;
output.mu=mu;
output.X=X;
