% this function illustrate the relationship between singluar value and the
% illumination.
% method
% 1. The image is normalized by taking zero mean and unit variance. 
%    However, according to the experiment l2 unit norm is better;
% 2. SVD is applied to the image and get U,S and V;
% 3. Reconstruct the image without first K singular value;
% 4. Illustrate the result;
% parameters:
% file1:    the image you want to process;
% file2:    the image you want to compare, can be null;
% K:        the cutting point, if K<0, do it interaction way;
function [limg rimg]=SVDImage(file1, file2, K)
% read in img 1
img1=imread(file1);
% convert to gray;
if size(img1, 3)>1
    img1=double(rgb2gray(img1));
end
% zero mean
% avg=mean(img1(:));
% img1=img1-avg;
% unit variance you may use unit l2 norm
% vari=std(img1(:));
% img1=img1/vari;
% svd of img1
img1=img1/norm(img1);
[u1 s1 v1]=svd(img1);
rs=min(size(s1));
if isempty(file2)
    % single image
    if K>0
        % mode 1
        K=min(K, rs);
        % limg
        tmp1=s1;
        tmp1(1:K, :)=0;
        limg=u1*tmp1*v1';
        % rimg
        tmp2=s1;
        tmp2(K+1:end, :)=0;
        rimg=u1*tmp2*v1';
        % draw
        subplot(121);imagesc(limg);axis image;colormap gray;
        title(printf('reconstructed from first %d singular value', K));
        subplot(122);imagesc(rimg);axis image;colormap gray;
        title('reconstructed from other singular value');
    else
        % interaction mode
        disp('press mouse for next change');
        K=1;
        tmp1=zeros(size(s1));
        tmp2=s1;
        while 1
            if waitforbuttonpress()==1 || K>rs
                break;
            end
            % limg
            tmp1(K, K)=1;
            limg=u1*tmp1*v1';
            % rimg
            tmp2(K, K)=0;
            rimg=u1*tmp2*v1';
            % draw
            subplot(121);imagesc(limg);axis image;colormap gray;
            title(printf('reconstructed from first %d singular value', K));
            subplot(122);imagesc(rimg);axis image;colormap gray;
            title('reconstructed from other singular value');
            K=K+1;
        end
    end
else
    % two image
    % read in img 1
    img2=imread(file2);
    % convert to gray;
    if size(img2, 3)>1
        img2=double(rgb2gray(img2));
    end
    % zero mean
%     avg=mean(img2(:));
%     img2=img2-avg;
    % unit variance you may use unit l2 norm
    % vari=std(img2(:));
    % img2=img2/vari;
    img2=img2/norm(img2);
    % svd of img1
    [u2 s2 v2]=svd(img2);
    if rs~=min(size(s2))
        disp('different size');
        exit;
    end
    if K>0
        % mode 1
        K=min(K, rs);
        % limg
        tmp1=s1;
        tmp1(1:K, :)=0;
        limg=u1*tmp1*v1';
        % rimg
        tmp2=s2;
        tmp2(1:K, :)=0;
        rimg=u2*tmp2*v2';
        % draw
        subplot(121);imagesc(limg);axis image;colormap gray;
        title(printf('image: %s \nreconstructed without first %d singular value', file1, K));
        subplot(122);imagesc(rimg);axis image;colormap gray;
        title(printf('image: %s \nreconstructed without first %d singular value', file2, K));
        % calculat the error
        err=norm(limg(:)-rimg(:))/max(norm(limg(:)), norm(rimg(:)));
        disp(sprintf('normalized error is %.4f', err));
    else
        % interaction mode
        K=1;
        tmp1=s1;
        tmp2=s2;
        while 1
            if waitforbuttonpress()==1 || K>rs
                break;
            end
            % limg
            tmp1(K, K)=0;
            limg=u1*tmp1*v1';
            % rimg
            tmp2(K, K)=0;
            rimg=u2*tmp2*v2';
            % draw
            subplot(121);imagesc(limg);axis image;colormap gray;
            title(sprintf('image: %s \nreconstructed without first %d singular value', file1, K));
            subplot(122);imagesc(rimg);axis image;colormap gray;
            title(sprintf('image: %s \nreconstructed without first %d singular value', file2, K));
            % calculat the error
            err=norm(limg(:)-rimg(:))/max(norm(limg(:)), norm(rimg(:)));
            disp(sprintf('normalized error is %.4f', err));
            K=K+1;
        end
    end
end