% V4:   Treat the subspace of male and female seperately;
% V3:   add a zero term to the subspace, i.e. the subspace is {X, {A_j}}
% and {C_i, {A_j}}
% V2:   the classification is based on the principle angle between two
% subspaces, i.e. C_i-{A_j} and X-{A_j}
% V1:   the classification is based on the principle angle between two
% subspaces, i.e. C_i+{A_j} and X+{A_j}
% parameters:
% file_name:    the image name for test;
% C_hat:        the common part, where each column is for one subject;
% A_hat:        the low rank part, where each column is for one condition;
% G_hat:        not used in this version;
% option:       a structure contains many parameter;
% =========================================================================
function [idx, output]=TestSubspace(file_name, C_hat, A_hat, G_hat, option)
%% ========================================================================
% preprocessing
% number of images
N=2;
% read in the image
img=imread(file_name);
% color image?
if size(img, 3)~=1
    img=0.2989*img(:, :, 1)+0.5870*img(:, :, 2)+0.1140*img(:, :, 3);
end
% resize the image?
if option.height~=0 && option.width~=0
    img=imresize(img, [option.height, option.width]);
end
height=size(img, 1);
width=size(img, 2);
% some preprocessing may be need here
img=double(img);
% normalization
switch option.mode
    case 0
        % do nothing
    case 1
        % unit l2-norm for each image
        img=img/norm(img, 'fro');
    case 2
        % zero mean and unit l2-norm for each image
        img=img-mean(img(:));
        img=img/norm(img, 'fro');
    case 3
        % zero mean and unit std for eac dimension
        img=img-mean(img(:));
        img=img/std(img(:));
    case 4
        % zerp mean
        img=img-mean(img(:));
end
%% ========================================================================
% Parameters
output.err=zeros(option.n_People, 1);
output.block=zeros(option.n_People, 25);
%% ========================================================================
if strcmpi(option.dataset, 'AR')
    % for AR we treat male and female seperately
    n_People=option.n_People/2;
    n_Condition=option.n_Condition/2;
    % first part
    X=repmat(img(:), 1, n_Condition)+A_hat(:, 1: n_Condition);
    output.coef=zeros(option.n_People, size(X, 2));
    for i=1: n_People
        output.coef(i, :)=subspacea(repmat(C_hat(:, i), 1, n_Condition)+...
            A_hat(:, 1: n_Condition), X);
        output.block(i, :)=BlockError(repmat(C_hat(:, i), 1, n_Condition)+...
            A_hat(:, 1: n_Condition), X, option);
    end
    % second part
    X=repmat(img(:), 1, n_Condition)+A_hat(:, 1+n_Condition: option.n_Condition);
    for i=1+n_People: option.n_People
        output.coef(i, :)=subspacea(repmat(C_hat(:, i), 1, n_Condition)+...
            A_hat(:, 1+n_Condition: option.n_Condition), X);
        output.block(i, :)=BlockError(repmat(C_hat(:, i), 1, n_Condition)+...
            A_hat(:, 1+n_Condition: option.n_Condition), X, option);
    end
elseif strcmpi(option.dataset, 'YaleB')
    X=repmat(img(:), 1, size(A_hat, 2))+A_hat;
    output.coef=zeros(option.n_People, size(X, 2));
    for i=1: option.n_People
        output.coef(i, :)=subspacea(repmat(C_hat(:, i), 1, size(A_hat, 2))+...
            A_hat, X);
        output.block(i, :)=BlockError(repmat(C_hat(:, i), 1, size(A_hat, 2))+...
            A_hat, X, option);
    end
end
%% ========================================================================
% do classification according to the reconstruction error
output.err=sum(cos(output.coef).^2, 2);
[val, idx]=max(output.err);
tmp=max(output.block);
output.block=output.block./repmat(tmp, option.n_People, 1);
output.block=sum(output.block, 2);
[val, output.idx]=max(output.block);
end

% calculate the error in each block (1/2 overlapping), where mode decides 
% the error measurement we use:
% 1:    l1 norm;
% 2:    l2 norm;
function err=BlockError(X, Y, option)
err=zeros(1, 25);
nCol=size(X, 2);
X=reshape(X, option.height, []);
Y=reshape(Y, option.height, []);
tmpX=[];
tmpY=[];
tmpTheta=[];
l=1;
if strcmpi(option.dataset, 'AR')
    for i=[1 10 19 28 38]
        for j=[1 8 15 22 27]
            for k=1: nCol
                tmpX(:, k)=reshape(X(i: i+17, (k-1)*option.width+(j: j+13)), [], 1);
                tmpY(:, k)=reshape(Y(i: i+17, (k-1)*option.width+(j: j+13)), [], 1);
            end
            tmpTheta=subspacea(tmpX, tmpY);
            err(l)=sum(cos(tmpTheta).^2);
            l=l+1;
        end
    end
elseif strcmpi(option.dataset, 'YaleB')
    for i=[1 9 17 25 33]
        for j=[1 8 15 22 29]
            for k=1: nCol
                tmpX(:, k)=reshape(X(i: i+15, (k-1)*option.width+(j: j+13)), [], 1);
                tmpY(:, k)=reshape(Y(i: i+15, (k-1)*option.width+(j: j+13)), [], 1);
            end
            tmpTheta=subspacea(tmpX, tmpY);
            err(l)=sum(cos(tmpTheta).^2);
            l=l+1;
        end
    end
end
end