% this function implement extendSRC approach
% Input:
%   x:      [dx1] test image;
%   D1:     [dxn] dictionary of the face image;
%   D2:     [dxm] dictionary of variations;
%   gamma:  [1x1] the weight for the sparsity term;
%   w1:     [1x1] or [1xn] the weight of coefficient for the atoms of face
%       images;
%   w2:     [1x1] or [1xn] the weight of coefficient for the atoms of
%       dictionary atoms;
%   use_cplex:  [1x1] whether to use cplex as QP optimizer;
% Note:
%   1:      gamma, w1 and w2 should be (non)negative;
function [idx x_err y]=extend_src(x, D1, D2, gamma, w1, w2, train_id, Q, A, b, lb, ub)
    use_cplex=0;
    % input
    d=size(x, 1);
    n=size(D1, 2);
    m=size(D2, 2);
    D1tx=D1'*x;
    D2tx=D2'*x;
    gamma=gamma*max(max(D1tx), max(D2tx));
    w1=w1*gamma;
    w2=w2*gamma;
    if length(w1)==1
        w1=w1*ones(n, 1);
    end
    if length(w2)==1
        w2=w2*ones(m, 1);
    end
    % construct the formulations
%     D1tD2=D1'*D2;
%     Q=[ D1'*D1, D1tD2,  zeros(n, m);...
%         D1tD2', D2'*D2,  zeros(m, m);...
%         zeros(m, 2*m+n)];
    f=[w1-D1tx; -D2tx; w2];
%     A=[ zeros(m, n),    -eye(m),    -eye(m);...
%         zeros(m, n),    eye(m),     -eye(m)];
%     b=zeros(2*m, 1);
%     lb=[zeros(n, 1); -inf(m, 1); zeros(m, 1)];
%     ub=inf(2*m+n, 1);
    % apply the optimization
    if use_cplex
        % cplex
        [y,fval,exitflag,output,lambda]=cplexqp(Q,f,A,b,[],[],lb,ub);
    else
        % matlab default one
        option=optimset('MSK_IPAR_INTPNT_NUM_THREADS', 0);
        % option=optimset('Algorithm', 'interior-point-convex');
        [y,fval,exitflag,output,lambda]=quadprog(Q,f,A,b,[],[],lb,ub, [], option);
    end
    % do the classification
    x_res=x-D2*y(n+1: n+m);
    number_subject=length(train_id);
    x_err=zeros(number_subject, 1);
    for i=1: number_subject
        x_err(i)=norm(x_res-D1(:, train_id{i})*y(train_id{i}));
    end
    [val idx]=min(x_err);
    y=y(1: n+m);
end