function [s] = subspace_linear(F,G)
%SUBSPACEA angles between subspaces with inner producte


threshold=0.707; % Define threshold for determining when an angle is small

if size(F,1) ~= size(G,1)
   subspaceaError(['The row dimension ' int2str(size(F,1)) ...
         ' of the matrix F is not the same as ' int2str(size(G,1)) ...
         ' the row dimension of G'])
end

%% A not available  
   % Trivial column scaling first, if ORTH.m is used later 
   for i=1:size(F,2),
     normi=norm(F(:,i),inf);
     %Adjustment makes tol consistent with experimental results
     if normi > 4.4041e-016
       F(:,i)=F(:,i)/normi;
       % Else orth will take care of this
     end
   end
   for i=1:size(G,2),
     normi=norm(G(:,i),inf);
     %Adjustment makes tol consistent with experimental results
     if normi > 4.4041e-016
       G(:,i)=G(:,i)/normi;
       % Else orth will take care of this
     end
   end
% Compute angle using standard inner product

QF = orth(F);      %This can also be done using QR.m, in which case
QG = orth(G);      %the column scaling above is not needed 

% q = min(size(QF,2),size(QG,2));
[Ys,s,Zs] = svd(QF'*QG,0);
if size(s,1)==1
    % make sure s is column for output
    s=s(1);
end
s = min(diag(s),1);
end

% this is no fast then SVD implementation
function Q = orth_fast(A)
% QR decomposition
[Q,R,E]=qr(A);
% Determine r = effective rank
tol = eps*norm(A,'fro');
r = sum(abs(diag(R)) > tol);
r = r(1); % fix for case where R is vector.
% Use first r columns of Q.
if r > 0
    Q = Q(:,1:r);
    % Cosmetic sign adjustment
    Q = -Q;
    Q(:,r) = -Q(:,r);
else
    Q = [];
end
end