// this file is the c++ implementation of subspaceaa
// to complie in windows
// mex('subspacea.cpp',...
// fullfile(matlabroot, 'extern', 'lib', 'win64', 'microsoft', 'libmwlapack.lib'),...
// fullfile(matlabroot, 'extern', 'lib', 'win64', 'microsoft', 'libmwblas.lib'));
// to complie in linux
// mex subspacea.cpp -lmwblas -lmwlapack
#if !defined(_WIN32)
#define dgemm dgemm_
#define dgesvd dgesvd_
#endif

#include "mex.h"
#include "blas.h"
#include "lapack.h"
#include <math.h>
#include <string.h>
// ========================================================================
// matrix production
void mm_lapack(double *A, double *B, double *C, ptrdiff_t D, ptrdiff_t M, ptrdiff_t N);
// ========================================================================
// orth decomposition with SVD
double * orth_lapack(double *A, int * K, ptrdiff_t M, ptrdiff_t N);
// ========================================================================
// svd with lapack
void svd_lapack(double *A, double *U, double *S, double *Vt, ptrdiff_t M, ptrdiff_t N);
// ========================================================================
// compute infinite norm for vector
double norm_inf(double *data, int D, double inf);
// ========================================================================
// the main function
// Input
//  F:      dxm matrix;
//  G:      dxn matrix;
// Output
//  theta:  1xmin(m,n) array, the principle angle;
//  U:      dxmin(m,n) matrix, the principle vector for F;
//  V:      dxmin(m,n) matrix, the principle vector for G;
void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    // ====================================================================
    // parse the input
    if (nrhs<2)
    {
        mexErrMsgTxt("Usage: theta=subspace(F, G)\n");
    }
    if (mxGetM(prhs[0])!=mxGetM(prhs[1]))
    {
        mexErrMsgTxt("The first dimension of F and G must match\n");
    }
    if (nlhs!=1)
    {
        mexErrMsgTxt("Usage: theta=subspace(F, G)\n");
    }
    // define the constant
    double threshold=sqrt(2.0)/2;
    double eps=mxGetEps();
    double eps_981=pow(eps, 0.981);
    double inf=mxGetInf();
    double pi_2=1.57;
    // ====================================================================
    // variable
    double *F, *G;
    F=mxGetPr(prhs[0]);
    G=mxGetPr(prhs[1]);
    int D, M, N;
    D=mxGetM(prhs[0]);
    M=mxGetN(prhs[0]);
    N=mxGetN(prhs[1]);
    double vector_max;
    int mn_min, mn_max;
    if(M>N)
    {
        mn_min=N;
        mn_max=M;
    }
    else
    {
        mn_min=M;
        mn_max=N;
    }
    int i, j, ij;
    double *QF, *QG;
    int QF_col, QG_col;
    double *QFt;
    double *QFtQG;
    double *Ys, *s, *Zs, *Zst;
    double *U, *V;
    double *RF, *RG;
    double *QF2RG, *RG_QF2RG;
    double tmp;
    double *zx;
    double *x;
    int nd_min;
    int q;
    // ====================================================================
    // Init
    for (i=0; i<M; i++)
    {
        vector_max=norm_inf(F+i*D, D, inf);
        if (vector_max>eps_981)
        {
            ij=i*D;
            for (j=0; j<D; j++)
            {
                F[ij+j]=F[ij+j]/vector_max;
            }
        }
    }
    for (i=0; i<N; i++)
    {
        vector_max=norm_inf(G+i*D, D, inf);
        if (vector_max>eps_981)
        {
            ij=i*D;
            for (j=0; j<D; j++)
            {
                G[ij+j]=G[ij+j]/vector_max;
            }
        }
    }
    // ====================================================================
    // coarse step
    // use lapack to compute QR
    QF=orth_lapack(F, &QF_col, (ptrdiff_t) D, (ptrdiff_t) M);
    QG=orth_lapack(G, &QG_col, (ptrdiff_t) D, (ptrdiff_t) N);
    if (QF_col>QG_col)
    {
        q=QG_col;
    }
    else
    {
        q=QF_col;
    }
    // compute the angle using inner product
    QFt=new double[D*QF_col];
    for (i=0; i<D; i++)
    {
        for (j=0; j<QF_col; j++)
        {
            QFt[i*QF_col+j]=QF[j*D+i];
        }
    }
    QFtQG=new double[QF_col*QG_col];
    mm_lapack(QFt, QG, QFtQG, (ptrdiff_t) D, (ptrdiff_t) QF_col, (ptrdiff_t) QG_col);
    // Ys=new double[QF_col*q];
    Zs=new double[QG_col*q];
    s=new double[q];
    // svd_lapack(QFtQG, Ys, s, Zs, QF_col, QG_col);
    svd_lapack(QFtQG, NULL, s, Zs, (ptrdiff_t) QF_col, (ptrdiff_t) QG_col);
	// transpose
	Zst=new double[QG_col*q];
	for (i=0; i<q; i++)
	{
		for (j=0; j<QG_col; j++)
		{
			Zst[i*QG_col+j]=Zs[i+j*q];
		}
	}
    // ====================================================================
    // refine step
    // U=new double[D*q];
    V=new double[D*q];
    // mm_lapack(QF, Ys, U, QF_col, D, q);
    mm_lapack(QG, Zst, V, (ptrdiff_t) QG_col, (ptrdiff_t) D, (ptrdiff_t) q);
    // refine the result
    int n_refined=0;
    for (i=0; i<q; i++)
    {
        if (s[i]>threshold)
        {
            n_refined++;
        }
    }
    if (n_refined>0)
    {
        // RF=new double[D*n_refined];
        RG=new double[D*n_refined];
        n_refined=0;
        for (i=0; i<q; i++)
        {
            if (s[i]>threshold)
            {
                // memcpy(RF+n_refined*D, U+D*i, D*sizeof(double));
                memcpy(RG+n_refined*D, V+D*i, D*sizeof(double));
                n_refined++;
            }
        }
        QF2RG=new double[QF_col*n_refined];
        mm_lapack(QFt, RG, QF2RG, (ptrdiff_t) D, (ptrdiff_t) QF_col, (ptrdiff_t) n_refined);
        RG_QF2RG=new double[D*n_refined];
        mm_lapack(QF, QF2RG, RG_QF2RG, (ptrdiff_t) QF_col, (ptrdiff_t) D, (ptrdiff_t) n_refined);
		for (i=0; i<D; i++)
		{
			for (j=0; j<n_refined; j++)
			{
				RG_QF2RG[i*n_refined+j]=RG[i*n_refined+j]-RG_QF2RG[i*n_refined+j];
			}
		}
        if (D>n_refined)
        {
            nd_min=n_refined;
        }
        else
        {
            n_refined=nd_min;
        }
        x=new double[nd_min];
        svd_lapack(RG_QF2RG, NULL, x, NULL, (ptrdiff_t) D, (ptrdiff_t) n_refined);
        // we only care about the angles
        n_refined=0;
        for (i=0; i<q; i++)
        {
            if (s[i]>threshold)
            {
                tmp=x[nd_min-n_refined-1];
                tmp=tmp<1? tmp: 1;
                tmp=asin(tmp);
                tmp=tmp>0? tmp: 0;
                s[i]=tmp;
                n_refined++;
            }
            else
            {
                tmp=acos(s[i]);
                s[i]=tmp>0? tmp: 0;
            }
        }
    }
    // ====================================================================
    // prepapre the output
    plhs[0]=mxCreateDoubleMatrix(mn_min, 1, mxREAL);
    double *theta=mxGetPr(plhs[0]);
    for (i=0; i<q; i++)
    {
        theta[i]=s[i];
    }
    for (i=q; i<mn_min; i++)
    {
        theta[i]=pi_2;
    }
    // ====================================================================
    // clean up
	// there is some potential memory leakage here
    delete []QF;
    delete []QG;
    delete []QFt;
    delete []QFtQG;
    // delete []U;
    delete []V;
    // delete []RF;
    delete []RG;
    delete []QF2RG;
    delete []RG_QF2RG;
    // delete []zx;
    // delete []Ys;
    delete []Zs;
	delete []Zst;
    delete []s;
    delete []x;
}
// ========================================================================
// compute infinite norm for vector
double norm_inf(double *data, int D, double inf)
{
    double max=-inf, tmp;
    int i;
    for (i=0; i<D; i++)
    {
		tmp=abs(data[i]);
        if(max<tmp)
        {
            max=tmp;
        }
    }
    return max;
}
// ========================================================================
// svd with lapack
void svd_lapack(double *A, double *U, double *S, double *Vt, ptrdiff_t M, ptrdiff_t N)
{
    // first make a copy of A
    int MN=M*N;
	bool U_req, V_req;
    double *X=new double[MN];
    memcpy(X, A, MN*sizeof(double));
    // create the workspace
    ptrdiff_t d_min=0, d_max=0;
    if (M>N)
    {
        d_max=M;
        d_min=N;
    }
    else
    {
        d_max=N;
        d_min=M;
    }
    ptrdiff_t lwork=4*d_max*d_max;
    if (lwork<10)
    {
        lwork=10;
    }
    double *work=new double[lwork];
    // check the status
    char jobu, jobvt;
    if (Vt==NULL && U==NULL)
    {
        jobvt='N';
        jobu='N';
    }
    else
    {
        jobvt='S';
        jobu='S';
    }
//    jobvt='A';
//    jobu='A';
    if (Vt==NULL)
    {
        // Vt is not required
        V_req=0;
		Vt=new double[d_min*N];
    }
    else
    {
		V_req=1;
    }
    if (U==NULL)
    {
        // Vt is not required
        U_req=0;
		U=new double[d_min*M];
    }
    else
    {
		U_req=1;
    }    
	// call the routine
    ptrdiff_t info;
    dgesvd(&jobu, &jobvt, &M, &N, X, &M, S, U, &M, Vt, &d_min, work, &lwork, &info);
    // clean the stuff
    if (info)
    {
        mexErrMsgTxt("SVD computation error\n");
    }
    delete []work;
    delete []X;
	if (!U_req)
	{
		delete []U;
	}
	if (!V_req)
	{
		delete []Vt;
	}
}
// ========================================================================
// orth decomposition with SVD
double * orth_lapack(double *A, int *K, ptrdiff_t M, ptrdiff_t N)
{
    // create variable
    int d_min, d_max;
	double *Q;
    if (M>N)
    {
        d_min=N;
        d_max=M;
    }
    else
    {
        d_min=M;
        d_max=N;
    }
    // svd
    double *S=new double[d_min];
    double *U=new double[M*d_min];
    svd_lapack(A, U, S, NULL, M, N);
    // check the valid columns
    double tol=-mxGetInf();
    int i;
    for (i=0; i<d_min; i++)
    {
        if (tol<S[i])
        {
            tol=S[i];
        }
    }
    tol=d_max*tol*mxGetEps();
    int n_valid=0;
    for (i=0; i<d_min; i++)
    {
        if (S[i]>tol)
        {
            n_valid++;
        }
        else
        {
            break;
        }
    }
    Q=new double[M*n_valid];
    memcpy(Q, U, M*n_valid*sizeof(double));
    delete []S;
    delete []U;
	*K=n_valid;
    return Q;
}
// ========================================================================
// matrix production
//  A:  MxD;
//  B:  DxN;
//  C:  MxN;
void mm_lapack(double *A, double *B, double *C, ptrdiff_t D, ptrdiff_t M, ptrdiff_t N)
{
    char transA, transB;
    transA='N';
    transB='N';
    double alpha, beta;
    alpha=1.0;
    beta=0;
    dgemm(&transA, &transB, &M, &N, &D, &alpha, A, &M, B, &D, &beta, C, &M);
}